#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: __init__.py
@Time: 2021/6/21 16:26
'''


import os
import sys
import json
import argparse

import grequests
import requests
import configparser

from kunyu.utils import *
from kunyu.config import setting
from kunyu.config.__version__ import usage, init, __title__, __help__

parser = argparse.ArgumentParser(prog=__title__)

# console pattern subcommand
subparsers = parser.add_subparsers()
parser_group_console = subparsers.add_parser('console', help='Enter Console Mode',
                                             description=__help__.format(datil=usage),
                                             formatter_class=argparse.RawDescriptionHelpFormatter,
                                             usage=argparse.SUPPRESS, add_help=True)

# Initial Configuration
parser_init_console = subparsers.add_parser('init', help='Enter init information',
                                            description=__help__.format(datil=init),
                                            formatter_class=argparse.RawDescriptionHelpFormatter,
                                            usage=argparse.SUPPRESS, add_help=True)


parser_init_console.add_argument("--seebug", help='Seebug API Key')
parser_init_console.add_argument("--apikey", help='ZoomEye API Key')
parser_init_console.add_argument("--username", help='ZoomEye Username')
parser_init_console.add_argument("--password", help='ZoomEye Password')
parser_init_console.add_argument("--rule", help='Set Rule File Path')
parser_init_console.add_argument("--output", help='Set Output File Path')
parser_init_console.add_argument("--serverless", help='Set Serverless API')

args = parser.parse_args()

# Gets the absolute path of the project
path = os.path.expanduser("~/")
conf = configparser.ConfigParser()

__path = os.path.join(path, ".kunyu.ini")

# Read config.user.ini
conf.read(__path)

def initial_config():
    """
        Determine whether the parameters in the configuration file exist
        If it does not exist, create a parameter and set the initial value to None
    """
    if not conf.has_section("zoomeye") and not conf.has_section("login"):
        conf.add_section('zoomeye')
        conf.set("zoomeye", "apikey", "None")
        conf.add_section('login')
        conf.set("login", "token", "None")

    if not conf.has_section("seebug"):
        conf.add_section('seebug')
        conf.set("seebug", "apikey", "None")

    # The path of the output file
    if not conf.has_section("path"):
        conf.add_section("path")
        conf.set("path", "output", setting.OUTPUT_PATH)

    # Set Serverless API Address Config
    if not conf.has_section("Serverapi"):
        conf.add_section("Serverapi")
        conf.set("Serverapi", "serverless", "None")

    if not conf.has_section("rule"):
        conf.add_section("rule")
        conf.set("rule", "path", setting.RULE_FILE_PATH)

# Verify the login status of the ZoomEye account
def _get_login():
    param = '{{"username": "{}", "password": "{}"}}'.format(args.username, args.password)
    resp = requests.post(
        "https://api.zoomeye.org/user/login",
        data=param,
        timeout=15
    )
    resp = json.loads(resp.text)
    try:
        if resp["error"]:
            raise requests.HTTPError(resp.get("message"))
    except:
        pass
    conf.set("login", "token", resp["access_token"])


try:
    # Initialize the configuration file
    initial_config()
    if args.apikey:
        conf.set("zoomeye", "apikey", args.apikey)

    elif args.username and args.password:
        _get_login()

    if args.seebug:
        conf.set("seebug", "apikey", args.seebug)

    # set output file path
    if args.output:
        conf.set("path", "output", args.output)

    # Used for CrashHost function
    if args.serverless:
        conf.set("Serverapi", "serverless", args.serverless)

    # Used for rule directory path
    if args.rule:
        conf.set("rule", "path", args.rule)

except requests.HTTPError as err:
    print("\033[31;1m{}\033[0m".format(err))
    print(__help__.format(datil=init))
    sys.exit(0)

except Exception:
    pass

with open(__path, "w+") as f:
    conf.write(f)








#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: keyword.py
@Time: 2021/11/12 9:23
'''
import re

from rich.table import Table
from rich.console import Console

console = Console(color_system="auto", record=True)
EMAIL_REGEX = r"([\w-]+(\.[\w-]+)*@[\w-]+(\.[\w-]+)+)"
PHONE_REGEX = r"^1[358]\d{9}$|^147\d{8}$|^179\d{8}$"
DOMAIN_REGEX = r'<a\s*href=\"(.*?)\"'
IDENTITY_REGEX = r"[1-9]\d{5}(?:18|19|(?:[23]\d))\d{2}(?:(?:0[1-9])|(?:10|11|12))(?:(?:[0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]"
IP_ADDRESS_REGEX = r"\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b"
DOMAIN_CHECK_REGEX = r'<a\s*href=\"(.*?)\"'


class SearchKeyWord:
    def __init__(self):
        self.sensitive_params = []

    def __check_email(self, banner):
        email_mod = re.compile(EMAIL_REGEX)
        items = email_mod.findall(banner)
        return items

    # Check whether it is valid phone number
    def __check_phone(self, banner):
        phone_mod = re.compile(PHONE_REGEX)
        items = phone_mod.findall(banner)
        return items

    """
        # Check whether it is valid domain address
        def __check_domain_address(self, banner):
            domain_mod = re.compile(DOMAIN_REGEX)
            items = domain_mod.findall(banner)
            return items
    """

    # Check whether it is valid ip address
    def __check_ip_address(self, banner):
        ip_mod = re.compile(IP_ADDRESS_REGEX)
        items = ip_mod.findall(banner)
        return items

    """
        # Check whether it is valid identity card
        def __check_identity(self, banner):
            identity_mod = re.compile(IDENTITY_REGEX)
            items = identity_mod.findall(banner)
            return items
    """

    def get_keyword_sensitive(self, banner):
        # Obtain email information in the banner
        for email in self.__check_email(banner):
            self.sensitive_params.append(email[0])

        # Obtain phone number information in the banner
        for phone in self.__check_phone(banner):
            self.sensitive_params.append(phone[0])

        """ 
            # Obtain domain information in the banner
            for domain in self.__check_domain_address(banner):
                self.sensitive_params.append(domain[0])

            # Obtain identity card information in the banner
            for identity in self.__check_identity(banner):
                self.sensitive_params.append(identity[0])
        """

        # Obtain ip address information in the banner
        for ip in self.__check_ip_address(banner):
            self.sensitive_params.append(ip)

        if self.sensitive_params:
            return self.sensitive_params



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: export.py
@Time: 2021/6/22 15:53
'''

import os
import datetime

import csv
import xlwt

from kunyu.core import conf
from kunyu.utils.log import logger
from kunyu.config import setting

""""
    According to different search engines,
    Select the appropriate ICO icon encryption method.
    ZoomEye, for example, supports both MMH3 and MD5,But FoFa only supports MMH3.
    Through the ICO icon search Related assets,It's very always efficient.
    Security researchers can also modify code files as needed.
"""

OUTPUT_PATH = conf.get("path", "output")


def createdir():
    # Create the results output directory.
    __dirnamae = datetime.datetime.now().strftime("%Y%m%d%H%M")
    path = os.path.expanduser(OUTPUT_PATH)
    __path = os.path.join(path, __dirnamae)
    setting.OUTPUT_PATH = __path
    if os.path.exists(__path):
        return 0
    # Create Directory
    os.makedirs(__path)


def export_csv(columns, data):
    # The export file format is CSV
    __filename = datetime.datetime.now().strftime("%H%M%S.csv")
    path = os.path.join(setting.OUTPUT_PATH, __filename)
    try:
        with open(path, "a", newline='', encoding="utf-8-sig") as f:
            writer = csv.writer(f)
            writer.writerow(columns)
            for num, key in enumerate(data["results"]):
                writer.writerow([str(num + 1), key[0], key[1], key[2], key[3], key[4], key[5], key[6]])

    except Exception:
        logger.error("hope you don't Batch scanning")
    logger.info("File export Path:" + path)


def export_xls(content, head):
    style_head = xlwt.XFStyle()
    font = xlwt.Font()
    font.name = "微软雅黑"  # Set title font
    font.bold = True
    font.height = 250  # Set title height
    font.colour_index = 1
    bg = xlwt.Pattern()
    bg.pattern = xlwt.Pattern.SOLID_PATTERN
    bg.pattern_fore_colour = 0x00
    al = xlwt.Alignment()
    al.horz = 0x02  # Set horizontal center
    style_head.font = font
    style_head.pattern = bg
    style_head.alignment = al
    excel = xlwt.Workbook(encoding='utf-8')
    sheet = excel.add_sheet("坤舆")
    for i in range(10):
        first_col_1 = sheet.col(i)
        first_col_1.width = 256 * 20
    for index, value in enumerate(head):
        sheet.write(0, index, value, style_head)

    # Set the cell format
    style = xlwt.XFStyle()
    borders = xlwt.Borders()
    borders.left = 1
    borders.right = 1
    borders.top = 1
    borders.bottom = 1
    borders.left_colour = 0
    borders.right_colour = 0
    borders.top_colour = 0
    borders.bottom_colour = 0
    style.alignment = al
    style.borders = borders

    # Cyclic write
    for index, value_list in enumerate(content, 1):
        for i, value in enumerate(value_list):
            sheet.write(index, i, value, style)

    # Save excel file
    __time = datetime.datetime.now().strftime("%H%M%S.xls")
    file_name = excel.save(os.path.join(setting.OUTPUT_PATH, __time))
    return file_name


def export_txt(console):
    # The export file format is TXT
    __time = datetime.datetime.now().strftime("%H%M%S.txt")
    path = os.path.join(setting.OUTPUT_PATH, __time)
    console.save_text(path)


if __name__ == "__main__":
    createdir()



#!/usr/bin/env python
# encoding: utf-8
"""
@author: 风起
@contact: onlyzaliks@gmail.com
@File: rule.py
@Time: 2021/12/28 17:56
"""
import os
import yaml

from rich.table import Table
from rich.console import Console

from kunyu.core import conf
from kunyu.config.setting import RULE_INFO

console = Console(color_system="auto", record=True)
RULE_FILE_PATH = conf.get("rule", "path")

class YamlRule:
    def _all_path(self, dirname):
        result = []
        # Returns the paths to all files in the specified directory
        for maindir, subdir, file_name_list in os.walk(dirname):
            for filename in file_name_list:
                result.append(os.path.join(maindir, filename))  # Merge into a full path
        return result

    def _get_yaml_file(self, filename):
        # Read the information in the yaml file
        with open(filename, encoding="utf-8") as file:
            file_data = file.read()
        # Convert yaml file content to dictionary type
        return yaml.load(file_data, Loader=yaml.FullLoader)

    def get_yaml_list(self):
        yaml_params = []
        for file in self._all_path(RULE_FILE_PATH):
            yaml_params.append(self._get_yaml_file(file))
        return yaml_params


#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: only.py
@Time: 2021/12/28 16:14
'''
import os

import yaml
from rich.console import Console

from kunyu.utils.log import logger

console = Console(color_system="auto", record=True)

logger.info("Generate fingerprint rule files:")

try:
    input_kxid = input("\033[32;32m请输入KXID信息:\033[0m")
    input_author = input("\033[32;32m请输入作者名称:\033[0m")
    input_kx_name = input("\033[32;32m请输入指纹名称:\033[0m")
    input_description = input("\033[32;32m请输入指纹简介:\033[0m")
    input_kx_query = input("\033[32;32m请输入指纹信息:\033[0m")
    input_createdate = input("\033[32;32m请输入指纹创建时间:\033[0m")
    input_source = input("\033[32;32m请输入指纹来源:\033[0m")

    query = '''{finger}'''.format(finger=input_kx_query)

    file_data = {
        "KXID": input_kxid,
        "author": input_author,
        "kx_name": input_kx_name,
        "description": input_description,
        "kx_query": query,
        "createDate": input_createdate,
        "source": input_source
    }
    logger.info("The content of the fingerprint rule file is as follows:")
    # Convert to YAML data
    data = yaml.dump(file_data, allow_unicode=True)
    # print yaml file data information
    console.print(f"{data}")

    # Gets the absolute path to the build file
    file_path = f"{os.path.abspath(os.path.dirname(os.path.dirname(__file__)))}/rule/{input_kxid}.yaml"
    with open(file_path, "w", encoding="utf-8") as file:
        file.write(data)

    logger.info("A fingerprint rule file has been generated:" + file_path)
except:
    print("")
    logger.warning("Exit!")

